#ifndef CFCGI_LIB_HTTP_CLIENT_H
#define CFCGI_LIB_HTTP_CLIENT_H

#include <string>
#include <stdarg.h>
#include <stdio.h>
#include <memory>

#define FCGI_FORMAT_STR(dest,fmt,v) do {\
        char* p = NULL; \
        size_t sz = 0; \
        va_start(v, fmt); \
        sz = vsnprintf(p, sz, fmt, v); \
        va_end(v); \
        ++sz; \
        dest.reserve(sz); \
        dest.resize(sz); \
        p = const_cast<char *>(dest.c_str()); \
        va_start(v, fmt); \
        vsnprintf(p, sz, fmt, v); \
        va_end(v); \
        dest.resize(sz-1); \
    } while (0)

class FcgiContext;
// Http Client 暂时不支持 HTTP 2.0
class FcgiBaseHttp
{
public:
    FcgiBaseHttp(FcgiContext& ctx);
    virtual ~FcgiBaseHttp();
    virtual int HttpGet(std::string& resp, const char* req, ...) = 0;
    virtual int HttpPost(std::string& resp, const char* addr, const char* argFmt, ...) = 0;

    // Add Http Headers
    virtual void AddHeaders(const char* fmt, ...) = 0;

    // Add Http header
    virtual void AddHeader(const char* name, const char* value) = 0;
protected:
    FcgiContext& m_ctx;
};

typedef std::shared_ptr<FcgiBaseHttp> FcgiBaseHttpPtr;

class FcgiBaseHttpGateway
{
public:
    enum FCGI_HTTP_CLIENT_TYPE {
        FCGI_CLIENT_HTTP,
        FCGI_CLIENT_HTTPS
    };
    FcgiBaseHttpGateway();
    virtual ~FcgiBaseHttpGateway();
    static FcgiBaseHttpPtr GetClient(FcgiContext& ctx, FCGI_HTTP_CLIENT_TYPE type);
protected:
    virtual FcgiBaseHttpPtr GetHttpClient(FcgiContext& ctx) = 0;
    virtual FcgiBaseHttpPtr GetHttpsClient(FcgiContext& ctx) = 0;

    static FcgiBaseHttpGateway* m_me;
};

#endif // CFCGI_LIB_HTTP_CLIENT_H
