#ifndef CFCGI_LIB_REQEUST_H
#define CFCGI_LIB_REQEUST_H

#include <string>
#include <map>
#if __cplusplus > 201103L
#include <variant>
using std::variant;
#else
#include <boost/variant.hpp>
using boost::variant;
#endif
#include "cfcgi_cookie.h"
#include "cfcgi_form_data.h"

using fcgiValue = variant<unsigned int, std::string>;

struct FcgiValue {
    fcgiValue m_value;
    FcgiValue(const fcgiValue& v);


#if __cplusplus > 201103L
    unsigned int AsInt() {
        if (m_value.index() == 0) {
            return std::get<unsigned int>(m_value);
        }
        return 0;
    }

    std::string AsString() {
        if (m_value.index() == 1) {
            return std::get<std::string>(m_value);
        }
        return "";
    }
#else
    unsigned int AsInt() {
        unsigned int* pui = boost::get<unsigned int>(&m_value);
        if (pui) {
            return *pui;
        }
        return 0;
    }

    std::string AsString() {
        std::string* ps = boost::get<std::string>(&m_value);
        if (ps) {
            return *ps;
        }
        return "";
    }
#endif
    operator std::string() {
        return AsString();
    }

    operator unsigned int() {
        return AsInt();
    }

};
extern FcgiValue NullFcgiValue;

enum _METHODS {
    GET,
    POST,
    PUT,
    DELETE,
    HEAD,
    OPTIONS,
    TRACE,
    PATCH
};

class FcgiRequest {
public:
    using ParamType = std::map<std::string, FcgiValue>;
    using ArgsType = std::map<std::string, std::string>;
    FcgiRequest();
    ~FcgiRequest();
public:
    // api 方法
    std::string& Uri();
    const std::string& Query(const std::string& name, const std::string& dft = "");
    const std::string& PostForm(const std::string& name, const std::string& dft = "");
    FcgiValue& Header(const std::string& name);
    std::string& Body();
    std::string& RemoteAddr();
    std::string& ContentType();
    const int& Method();
    unsigned int& ContentLength();
    FcgiCookie& Cookie();
    FormData& GetFormData();
    std::vector<FileChunk*> Files();
public:
    // 以下方法仅为内部使用
    void uri(const std::string&& uri);
    void header(const std::string&& head);
    void body(const std::string&& body, const std::string& tmpdir = "");
private:
    void parseArgs(ArgsType& dest, const std::string& args);
    void header(const std::string& name, std::string& value);
    void header(const std::string& name, unsigned int& value);
private:
    static std::map<std::string, int> m_methodMap;
    int         m_imethod;
    std::string m_uri;
    std::string m_args;
    std::string m_method;
    std::string m_contentType;
    std::string m_remoteAddr;
    unsigned int m_contentLength;
    ArgsType    m_queryArgs;
    ArgsType    m_postForm;
    ParamType   m_headers;
    std::string m_body;
    FcgiCookie  m_cookies;
    bool        m_isfile;
    std::string m_boundary;
    FormData    m_formdata;
};

#endif
