#ifndef CFCGI_LIB_ROUTER_H
#define CFCGI_LIB_ROUTER_H
#include <string>
#include <map>
#include <vector>
#include "cfcgi_context.h"

using FcgiHandlerPtr = void (*)(FcgiContext& ctx);

class FcgiRouter {
    using Routers = std::map<std::string, FcgiHandlerPtr>;
public:
    FcgiRouter();
    ~FcgiRouter();

    FcgiRouter& Get(const std::string& path, FcgiHandlerPtr handler);
    FcgiRouter& Post(const std::string& path, FcgiHandlerPtr handler);
    FcgiRouter& Head(const std::string& path, FcgiHandlerPtr handler);
    FcgiRouter& Options(const std::string& path, FcgiHandlerPtr handler);
    FcgiRouter& Put(const std::string& path, FcgiHandlerPtr handler);
    FcgiRouter& Delete(const std::string& path, FcgiHandlerPtr handler);
    FcgiRouter& Trace(const std::string& path, FcgiHandlerPtr handler);
    FcgiRouter& Patch(const std::string& path, FcgiHandlerPtr handler);
    FcgiRouter& Group(const std::string& group);

    FcgiHandlerPtr GetRoute(FcgiContext& ctx);

    void Clear();
    void GetAllPath(std::vector<std::string>& res);

private:
    void addRoute(Routers & route, const std::string& path, FcgiHandlerPtr handler);
    FcgiHandlerPtr getRoute(Routers& route, const std::string& path);
    void getRoutePath(Routers& route, std::vector<std::string>& res);
private:
    Routers m_get;
    Routers m_post;
    Routers m_head;
    Routers m_options;
    Routers m_put;
    Routers m_delete;
    Routers m_connect;
    Routers m_trace;
    Routers m_patch;
    std::string m_curGroup;
};

extern FcgiRouter *AppRouter;

FcgiRouter& Get(const std::string& path, FcgiHandlerPtr handler);
FcgiRouter& Post(const std::string& path, FcgiHandlerPtr handler);
FcgiRouter& Head(const std::string& path, FcgiHandlerPtr handler);
FcgiRouter& Options(const std::string& path, FcgiHandlerPtr handler);
FcgiRouter& Put(const std::string& path, FcgiHandlerPtr handler);
FcgiRouter& Delete(const std::string& path, FcgiHandlerPtr handler);
FcgiRouter& Trace(const std::string& path, FcgiHandlerPtr handler);
FcgiRouter& Patch(const std::string& path, FcgiHandlerPtr handler);
FcgiRouter& Group(const std::string& group);

#endif
