#ifndef FCGI_PACKET_H
#define FCGI_PACKET_H
#include <string>
#include "cfcgisock.h"
#include <memory>

#define FCGI_MAX_CONNS  "FCGI_MAX_CONNS"
#define FCGI_MAX_REQS   "FCGI_MAX_REQS"
#define FCGI_MPXS_CONNS "FCGI_MPXS_CONNS"

// -- not the fcgi protocol max content length
#define FCGI_MAX_CONTENT_LEN 1024
#define FCGI_MAX_PACKET_LEN 4096

#define FCGI_KEEP_CONN 1
#define FCGI_VERSION 1

#ifndef Uint8
typedef unsigned char Uint8;
#endif
#ifndef Uint16
typedef unsigned short Uint16;
#endif
#ifndef Uint32
typedef unsigned int Uint32;
#endif

#define FCGI_CALC_PADDINGS(ctlen) (8*(((ctlen)+7)/8) - (ctlen))

enum FcgiReqType {
    FCGI_INVALID_REQUEST = 0,
    FCGI_BEGIN_REQUEST,
    FCGI_ABORT_REQUEST,
    FCGI_END_REQUEST,
    FCGI_PARAMS,
    FCGI_STDIN,
    FCGI_STDOUT,
    FCGI_STDERR,
    FCGI_DATA,
    FCGI_GET_VALUES,
    FCGI_GET_VALUES_RESULT
};

enum FcgiRole {
    FCGI_RESPONDER  = 1,
    FCGI_AUTHORIZER = 2,
    FCGI_FILTER     = 3
};

enum FcgiProStatus {
    FCGI_REQUEST_COMPLETE,
    FCGI_CANT_MPX_CONN,
    FCGI_OVERLOAD,
    FCGI_UNKNOWN_ROLE
};

struct FcgiHeader {
  Uint8     m_version;
  Uint8     m_type;
  Uint16    m_requestId;
  Uint16    m_contentLength;
  Uint8     m_paddings;
  Uint8     m_reserved;

  void encode(std::string& dest);
  void encode(char dest[]);
  bool decode(std::string& s, size_t& off);
};
typedef std::shared_ptr<FcgiHeader> FcgiHeaderPtr;

struct FcgiBegin {
    Uint16  m_roleId;
    Uint8   m_flags;
    void encode(std::string& dest);
    bool decode(std::string& s, size_t& off);
};
typedef std::shared_ptr<FcgiBegin> FcgiBeginPtr;

struct FcgiEnd {
    Uint32  m_appStatus;
    Uint8   m_proStatus;
    void encode(std::string& dest);
    void encode(char buf[]);
    bool decode(std::string& s, size_t& off);
};
typedef std::shared_ptr<FcgiEnd> FcgiEndPtr;

struct FcgiMessage {
    int m_recved;
    FcgiHeader m_header;
    std::string m_body;

    FcgiMessage();

    error Recv(CFcgiSock& sock);
    error RecvStart(CFcgiSock& sock, bool needSkip = false, Uint8 lastChar = 1);
    error RecvCont(CFcgiSock& sock);
    error Send(CFcgiSock& sock);
    void encode(std::string& data);
    int LeftBytes();
};
typedef std::shared_ptr<FcgiMessage> FcgiMessagePtr;

void put8(std::string& dest, const Uint8& x);
void put16(std::string& dest, const Uint16& x);
void put32(std::string& dest, const Uint32& x);
void putNameValue(std::string& dest, const std::string& name, const std::string& value);
void put8n(std::string& dest, int n, Uint8 c);

void put8(char dest[], const Uint8& x);
void put16(char dest[], const Uint16& x);
void put32(char dest[], const Uint32& x);

Uint8 get8(const std::string& s, size_t& off);
Uint16 get16(const std::string& s, size_t& off);
Uint32 get32(const std::string& s, size_t& off);
bool getNameValue(const std::string& s, size_t& off, std::string& name, std::string& value);
extern FcgiMessagePtr InvalidMsg;

#endif // FCGI_PACKET_H
